@@ -5,14 +5,14 @@ from fcntl import ioctl
 from threading import Thread
 from operator import lt, gt
 
-from cameractrls import v4l2_capability, v4l2_format, v4l2_requestbuffers, v4l2_buffer
-from cameractrls import VIDIOC_QUERYCAP, VIDIOC_G_FMT, VIDIOC_S_FMT, VIDIOC_REQBUFS, VIDIOC_QUERYBUF, VIDIOC_QBUF, VIDIOC_DQBUF, VIDIOC_STREAMON, VIDIOC_STREAMOFF
+from cameractrls import v4l2_capability, v4l2_format, v4l2_streamparm, v4l2_requestbuffers, v4l2_buffer
+from cameractrls import VIDIOC_QUERYCAP, VIDIOC_G_FMT, VIDIOC_G_PARM, VIDIOC_S_PARM
+from cameractrls import VIDIOC_REQBUFS, VIDIOC_QUERYBUF, VIDIOC_QBUF, VIDIOC_DQBUF, VIDIOC_STREAMON, VIDIOC_STREAMOFF
 from cameractrls import V4L2_CAP_VIDEO_CAPTURE, V4L2_CAP_STREAMING, V4L2_MEMORY_MMAP, V4L2_BUF_TYPE_VIDEO_CAPTURE
 from cameractrls import V4L2_PIX_FMT_YUYV, V4L2_PIX_FMT_YVYU, V4L2_PIX_FMT_UYVY, V4L2_PIX_FMT_YU12, V4L2_PIX_FMT_YV12
 from cameractrls import V4L2_PIX_FMT_NV12, V4L2_PIX_FMT_NV21, V4L2_PIX_FMT_GREY
 from cameractrls import V4L2_PIX_FMT_RGB565, V4L2_PIX_FMT_RGB24, V4L2_PIX_FMT_BGR24, V4L2_PIX_FMT_RX24
 from cameractrls import V4L2_PIX_FMT_MJPEG, V4L2_PIX_FMT_JPEG
-from cameractrls import find_usb_ids_in_sysfs, KIYO_PRO_USB_ID
 
 sdl2lib = ctypes.util.find_library('SDL2-2.0')
 if sdl2lib is None:
@@ -75,6 +75,11 @@ SDL_CreateRenderer.restype = ctypes.c_void_p
 SDL_CreateRenderer.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_uint32]
 # SDL_Renderer * SDL_CreateRenderer(SDL_Window * window, int index, Uint32 flags);
 
+SDL_RenderGetLogicalSize = sdl2.SDL_RenderGetLogicalSize
+SDL_RenderGetLogicalSize.restype = None
+SDL_RenderGetLogicalSize.argtypes = [ctypes.c_void_p, ctypes.POINTER(ctypes.c_int), ctypes.POINTER(ctypes.c_int)]
+# void SDL_RenderGetLogicalSize(SDL_Renderer * renderer, int *w, int *h);
+
 SDL_RenderSetLogicalSize = sdl2.SDL_RenderSetLogicalSize
 SDL_RenderSetLogicalSize.restype = ctypes.c_int
 SDL_RenderSetLogicalSize.argtypes = [ctypes.c_void_p, ctypes.c_int, ctypes.c_int]
@@ -247,7 +252,7 @@ SDL_PAL_INFERNO_L = b'\
 \xf9\xfc\x9d\0\xfa\xfd\xa1\0\xfc\xff\xa4\0'
 SDL_PAL_INFERNO = (ctypes.c_uint8 * len(SDL_PAL_INFERNO_L))(*SDL_PAL_INFERNO_L)
 
-SLD_PAL_VIRIDIS_L = b'\
+SDL_PAL_VIRIDIS_L = b'\
 \x44\x01\x54\0\x44\x02\x56\0\x45\x04\x57\0\x45\x05\x59\0\x46\x07\x5a\0\x46\x08\x5c\0\x46\x0a\x5d\0\x46\x0b\x5e\0\x47\x0d\x60\0\x47\x0e\x61\0\x47\x10\x63\0\
 \x47\x11\x64\0\x47\x13\x65\0\x48\x14\x67\0\x48\x16\x68\0\x48\x17\x69\0\x48\x18\x6a\0\x48\x1a\x6c\0\x48\x1b\x6d\0\x48\x1c\x6e\0\x48\x1d\x6f\0\x48\x1f\x70\0\
 \x48\x20\x71\0\x48\x21\x73\0\x48\x23\x74\0\x48\x24\x75\0\x48\x25\x76\0\x48\x26\x77\0\x48\x28\x78\0\x48\x29\x79\0\x47\x2a\x7a\0\x47\x2c\x7a\0\x47\x2d\x7b\0\
@@ -272,7 +277,7 @@ SLD_PAL_VIRIDIS_L = b'\
 \xc0\xdf\x25\0\xc2\xdf\x23\0\xc5\xe0\x21\0\xc8\xe0\x20\0\xca\xe1\x1f\0\xcd\xe1\x1d\0\xd0\xe1\x1c\0\xd2\xe2\x1b\0\xd5\xe2\x1a\0\xd8\xe2\x19\0\xda\xe3\x19\0\
 \xdd\xe3\x18\0\xdf\xe3\x18\0\xe2\xe4\x18\0\xe5\xe4\x19\0\xe7\xe4\x19\0\xea\xe5\x1a\0\xec\xe5\x1b\0\xef\xe5\x1c\0\xf1\xe5\x1d\0\xf4\xe6\x1e\0\xf6\xe6\x20\0\
 \xf8\xe6\x21\0\xfb\xe7\x23\0\xfd\xe7\x25\0'
-SLD_PAL_VIRIDIS = (ctypes.c_uint8 * len(SLD_PAL_VIRIDIS_L))(*SLD_PAL_VIRIDIS_L)
+SDL_PAL_VIRIDIS = (ctypes.c_uint8 * len(SDL_PAL_VIRIDIS_L))(*SDL_PAL_VIRIDIS_L)
 
 # palettes from https://github.com/groupgets/GetThermal/blob/master/src/dataformatter.cpp
 # print('\\\n'.join(re.findall('.{0,154}', ''.join([f'\\x{x[0:2]}\\x{x[2:4]}\\x{x[4:6]}\\0' for x in re.findall('......', bytes(a).hex())]))))
@@ -334,7 +339,7 @@ SDL_PALS = {
     'none': SDL_PAL_GRAYSCALE,
     'grayscale': SDL_PAL_GRAYSCALE,
     'inferno': SDL_PAL_INFERNO,
-    'viridis': SLD_PAL_VIRIDIS,
+    'viridis': SDL_PAL_VIRIDIS,
     'ironblack': SDL_PAL_IRONBLACK,
     'rainbow': SDL_PAL_RAINBOW,
 }
@@ -474,18 +479,7 @@ class V4L2Camera(Thread):
 
     def init_device(self):
         cap = v4l2_capability()
-        fmt = v4l2_format()
-        fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE
-
         ioctl(self.fd, VIDIOC_QUERYCAP, cap)
-        ioctl(self.fd, VIDIOC_G_FMT, fmt)
-
-        # Razer Kiyo Pro needs an S_FMT to work
-        if find_usb_ids_in_sysfs(self.device) == KIYO_PRO_USB_ID:
-            try:
-                ioctl(self.fd, VIDIOC_S_FMT, fmt)
-            except Exception as e:
-                logging.warning(f'V4L2Camera: Can\'t set fmt: {e}')
 
         if not (cap.capabilities & V4L2_CAP_VIDEO_CAPTURE):
             logging.error(f'{self.device} is not a video capture device')
@@ -494,6 +488,23 @@ class V4L2Camera(Thread):
         if not (cap.capabilities & V4L2_CAP_STREAMING):
             logging.error(f'{self.device} does not support streaming i/o')
             sys.exit(3)
+    
+        fmt = v4l2_format()
+        fmt.type = V4L2_BUF_TYPE_VIDEO_CAPTURE
+        ioctl(self.fd, VIDIOC_G_FMT, fmt)
+    
+        parm = v4l2_streamparm()
+        parm.type = V4L2_BUF_TYPE_VIDEO_CAPTURE
+
+        # Razer Kiyo Pro and Microsoft Lifecam HD-3000 need
+        # to set FPS before first streaming
+        ioctl(self.fd, VIDIOC_G_PARM, parm)
+
+        try:
+            ioctl(self.fd, VIDIOC_S_PARM, parm)
+        except Exception as e:
+            logging.error(f'VIDIOC_S_PARM failed {self.device}: {e}')
+            sys.exit(3)
 
         self.width = fmt.fmt.pix.width
         self.height = fmt.fmt.pix.height
@@ -791,7 +802,7 @@ class SDLCameraWindow():
     def toggle_fullscreen(self):
         self.fullscreen = not self.fullscreen
         SDL_SetWindowFullscreen(self.window, SDL_WINDOW_FULLSCREEN_DESKTOP if self.fullscreen else 0)
-        self.match_window_to_rotation()
+        self.match_window_to_logical()
 
     def rotate(self, angle):
         self.angle += angle
@@ -809,23 +820,26 @@ class SDLCameraWindow():
             )
             if SDL_RenderSetLogicalSize(self.renderer, self.cam.height, self.cam.width) != 0:
                 logging.warning(f'SDL_RenderSetlogicalSize failed: {SDL_GetError()}')
-        self.match_window_to_rotation()
+        self.match_window_to_logical()
     
-    def match_window_to_rotation(self):
+    def match_window_to_logical(self):
         if self.fullscreen:
             return
 
-        w = ctypes.c_int()
-        h = ctypes.c_int()
-        SDL_GetWindowSize(self.window, w, h)
+        win_w = ctypes.c_int()
+        win_h = ctypes.c_int()
+        SDL_GetWindowSize(self.window, win_w, win_h)
 
-        op = gt if self.cam.width > self.cam.height else lt
-        if self.angle in [0, 180] and op(w.value, h.value):
-            return
-        if self.angle in [90, 270] and op(h.value, w.value):
-            return
+        logical_w = ctypes.c_int()
+        logical_h = ctypes.c_int()
+        SDL_RenderGetLogicalSize(self.renderer, logical_w, logical_h)
 
-        SDL_SetWindowSize(self.window, h, w)
+        if logical_w.value < logical_h.value and win_w.value < win_h.value:
+            return
+        if logical_w.value > logical_h.value and win_w.value > win_h.value:
+            return
+        
+        SDL_SetWindowSize(self.window, win_h, win_w)
 
     def mirror(self, flip):
         self.flip += flip
